/*
 * write_generic.c - functions to write generic format output
 *
 * Copyright (C) 1998, John C. Peterson <mailto:jaypee@netcom.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2. A copy is included in this distribution in the file
 * named "LICENSE".
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License, version 2, for more details.
 *
 * $Id: write_generic.c,v 1.1 1998/05/05 06:43:17 jcp Exp $
 *
 */

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "faaconv.h"

static char outbuf[BUF_LEN];

static bool_t write_generic_apt(char **tok_str);
static bool_t write_generic_fix(char **tok_str);
static bool_t write_generic_ils(char **tok_str);
static bool_t write_generic_nav(char **tok_str);
static bool_t write_generic_rwy(char **tok_str);

/* -------------------------------------------------------------------------- */

static bool_t write_generic_apt(char **tok_str)
{

  apt_t *rec = (void *) tok_str;

  char *p = outbuf;

  int nscan;
  int distance;
  int degrees;
  int lat_deg;
  int lon_deg;
  int minutes;
  double lat_min;
  double lon_min;
  double seconds;
  char lat_ns[2];
  char lon_ew[2];
  char var_ew[2];

  /* city, state code */

  rec->city = strcap(rec->city);
  SPRINTF(p,"%s", rec->city);
  SPRINTF(p,", %s", rec->state_post_office_code);

  *p = '\0';
  fprintf(fpoutput, "%s\n", outbuf);
  p = outbuf;

  /* airport name */

  SPRINTF(p,"  %s", rec->airport_name);

  /* airport id */

  if (option_k) {
    rec->airport_id = strkid(rec->airport_id);
  }

  SPRINTF(p," (%s)", rec->airport_id);

  /* distance and compass heading to central business district */

  nscan = sscanf(rec->distance_of_airport_from_business_district,
                 "%d", &distance);

  if (nscan == 1) {
    SPRINTF(p,"  %0d", distance);
    SPRINTF(p," %s", rec->direction_of_airport_from_business_district);
  } else {
    return FALSE;
  }

  /* latitude */

  nscan = sscanf(rec->ref_point_latitude, "%d-%d-%lf%1[NS]",
                 &lat_deg, &minutes, &seconds, lat_ns);

  if (nscan == 4) {
    sprintf(rec->ref_point_latitude,"%d-%d-%d%c",
              lat_deg, minutes, (int)seconds, lat_ns[0]);
    SPRINTF(p,"  %s", rec->ref_point_latitude);
  } else {
    return FALSE;
  }

  /* longitude */

  nscan = sscanf(rec->ref_point_longitude, "%d-%d-%lf%1[EW]",
                 &lon_deg, &minutes, &seconds, lon_ew);

  if (nscan == 4) {
    sprintf(rec->ref_point_longitude,"%d-%d-%d%c",
              lat_deg, minutes, (int)seconds, lon_ew[0]);
    SPRINTF(p," %s", rec->ref_point_longitude);
  } else {
    return FALSE;
  }

  /* magnetic variation */

  if (model_priority) {

    /* use the geomagnetic field model to compute the magnetic variation */

    model.latitude = ((*lat_ns == 'N')? +1.0: -1.0)*(lat_deg + lat_min/60.0);
    model.longitude = ((*lon_ew == 'E')? +1.0: -1.0)*(lon_deg + lon_min/60.0);
    if (sscanf(rec->airport_elevation, "%lf", &model.altitude) == 1) {
      model.altitude = FEET2KM(model.altitude);
    } else {
      model.altitude = 1.0;
    }

    if (field_model(&model)) {
      SPRINTF(p,"  %d", ROUND(fabs(model.dec)));
      SPRINTF(p,"%s", (model.dec > 0.0)? "E": "W");
    } else {
      return FALSE;
    }

  } else {

    /* get the magnetic variation from the database */

    nscan = sscanf(rec->mag_variation_and_direction_deg,
                   "%d%1[EW]", &degrees, var_ew);

    if (nscan == 2) {
      SPRINTF(p,"  %s", rec->mag_variation_and_direction_deg);
    } else {
      return FALSE;
    }

  }

  /* sectional chart */

  if (rec->aeronautical_sectional_chart_for_airport[0] != '\0') {
    SPRINTF(p,"  %s", strcap(rec->aeronautical_sectional_chart_for_airport));
  }

  *p = '\0';
  fprintf(fpoutput, "%s\n", outbuf);
  p = outbuf;

  /* airport elevation */

  SPRINTF(p,"  EL %s", rec->airport_elevation);

  /* fuel availability */

  if (rec->fuel_types_available_for_public_use[0] != '\0') {
    SPRINTF(p,"  FUEL %s", rec->fuel_types_available_for_public_use);
  }

  /* oxygen availability */

  if (rec->type_of_bulk_oxygen_available[0] != '\0') {
    SPRINTF(p,"  OX %s", rec->type_of_bulk_oxygen_available);
    if (rec->type_of_bottled_oxygen_available[0] != '\0') {
      SPRINTF(p,",%s", rec->type_of_bottled_oxygen_available);
    }
  } else {
    if (rec->type_of_bottled_oxygen_available[0] != '\0') {
      SPRINTF(p,"  OX %s", rec->type_of_bottled_oxygen_available);
    }
  }

  /* traffic pattern altitude */

  if (rec->traffic_pattern_altitude_feet_agl[0] != '\0') {
    SPRINTF(p,"  TPA %s", rec->traffic_pattern_altitude_feet_agl);
  }

  /* normal exit */

  *p = '\0';

  fprintf(fpoutput, "%s\n", outbuf);

  return TRUE;

}

/* -------------------------------------------------------------------------- */

static bool_t write_generic_fix(char **tok_str)
{

  return FALSE;

}

/* -------------------------------------------------------------------------- */

static bool_t write_generic_ils(char **tok_str)
{

  return FALSE;

}

/* -------------------------------------------------------------------------- */

static bool_t write_generic_nav(char **tok_str)
{

  return FALSE;

}

/* -------------------------------------------------------------------------- */

static bool_t write_generic_rwy(char **tok_str)
{

  return FALSE;

}

/* -------------------------------------------------------------------------- */

bool_t write_generic(char **tok_str)
{

  bool_t rc = FALSE;

  switch (db_type) {

    case DB_APT:
      rc = write_generic_apt(tok_str);
      break;

    case DB_FIX:
      rc = write_generic_fix(tok_str);
      break;

    case DB_ILS:
      rc = write_generic_ils(tok_str);
      break;

    case DB_NAV:
      rc = write_generic_nav(tok_str);
      break;

    case DB_RWY:
      rc = write_generic_rwy(tok_str);
      break;

  }

  /* return result */

  return rc;

}

/* -------------------------------------------------------------------------- */

